﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Web;

namespace igHierarchicalGrid_loader.Models
{
    public class NewViewModel
    {
        private DataSet dataSetModel;
        private List<Category> categories;
        private List<Product> products;

        public DataSet DataSetModel
        {
            get
            {
                if (this.dataSetModel == null)
                {
                    this.dataSetModel = this.GetEmployDepartmentDataSet();
                }
                return this.dataSetModel;
            }
        }

        public IQueryable<Category> Categories
        {
            get
            {
                if (categories == null)
                    categories = CreateCategories();

                return categories.AsQueryable<Category>();
            }
        }

        public IQueryable<Product> Products
        {
            get
            {
                if (products == null)
                    products = CreateProducts();

                return products.AsQueryable<Product>();
            }
        }

        #region DataSet Data

        private enum SampleTables
        {
            Departments,
            Employees,
            EmployeeOptions
        }

        private DataTable GetSampleTable(SampleTables table)
        {
            DataTable retVal;
            switch (table)
            {
                case SampleTables.Departments:
                    retVal = this.GetDepartments();
                    break;
                case SampleTables.Employees:
                    retVal = this.GetEmployees();
                    break;
                case SampleTables.EmployeeOptions:
                    retVal = this.GetEmployeeOptions();
                    break;
                default:
                    retVal = this.GetEmployees();
                    break;
            }
            return retVal;
        }

        public DataSet GetEmployDepartmentDataSet()
        {
            DataSet ds = new DataSet("EmployeeRecords");

            // Get the tables to use in the DataSet
            DataTable Employees = this.GetEmployees();
            DataTable Departments = this.GetDepartments();
            DataTable Options = this.GetEmployeeOptions();

            // Add the tables to the DataSet
            ds.Tables.Add(Departments);
            ds.Tables.Add(Employees);
            ds.Tables.Add(Options);

            // Create the relationship between the tables.
            ds.Relations.Add("DepartmentEmployee", Departments.Columns["DepartmentID"], Employees.Columns["Department"]);
            ds.Relations.Add("EmployeeOption", Employees.Columns["EmployeeID"], Options.Columns["Employee"]);
            return ds;
        }

        private DataTable GetDepartments()
        {
            DataTable dt = new DataTable("Departments");
            dt.Columns.Add("DepartmentID", typeof(int));
            dt.Columns.Add("DepartmentName", typeof(string));

            for (int i = 0; i < 8; i++)
            {
                dt.Rows.Add(new object[] { i, string.Format("Department {0}", i) });
            }

            dt.PrimaryKey = new DataColumn[] { dt.Columns["DepartmentID"] };

            return dt;
        }

        private DataTable GetEmployees()
        {
            return this.GetEmployees(24);
        }

        private DataTable GetEmployees(int count)
        {
            DataTable dtData = new DataTable("Employees");

            dtData.Columns.Add("EmployeeID", typeof(int));
            dtData.Columns.Add("Name", typeof(string));
            dtData.Columns.Add("OnSite", typeof(bool));
            dtData.Columns.Add("DateOfHire", typeof(DateTime));
            dtData.Columns.Add("Department", typeof(int));
            dtData.Columns.Add("Site", typeof(string));


            for (int i = 0; i < count; i++)
            {
                dtData.Rows.Add(new object[] { i, string.Format("Name {0}", i), (i % 2 == 0), DateTime.Today.Date.AddDays(-1 * i), i % 8, "http://www.google.com" });
            }

            dtData.PrimaryKey = new DataColumn[] { dtData.Columns["EmployeeID"] };

            return dtData;
        }

        private DataTable GetEmployeeOptions()
        {
            DataTable dtData = new DataTable("EmployeeOptions");

            dtData.Columns.Add("OptionID", typeof(int));
            dtData.Columns.Add("Employee", typeof(int));
            dtData.Columns.Add("Option", typeof(string));
            dtData.Columns.Add("Description", typeof(string));


            for (int i = 0; i < 96; i++)
            {
                dtData.Rows.Add(new object[] { i, i % 24, string.Format("Option {0}", i), String.Format("Option Description {0}", i) });
            }

            dtData.PrimaryKey = new DataColumn[] { dtData.Columns["OptionID"] };

            return dtData;
        }

        #endregion DataSet Data

        #region private methods

        private List<Category> CreateCategories()
        {
            return new List<Category>
            {
                new Category
                {
                    CategoryID = 1,
                    CategoryName = "Beverages",
                    Description = "Soft drinks, coffees, teas, beers, and ales"
                   
                },

                new Category
                {
                    CategoryID = 2,
                    CategoryName = "Condiments",
                    Description = "Sweet and savory sauces, relishes, spreads, and seasonings"
                },

                new Category
                {
                    CategoryID = 3,
                    CategoryName = "Confections",
                    Description = "Desserts, candies, and sweet breads",
                },

                new Category
                {
                    CategoryID = 4,
                    CategoryName = "Dairy Product",
                    Description = "Cheeses",
                },

                new Category
                {
                    CategoryID = 5,
                    CategoryName = "Grains/Cereals",
                    Description = "Breads, crackers, pasta, and cereal",
                },

                new Category
                {
                    CategoryID = 6,
                    CategoryName = "Meat/Poultry",
                    Description = "Prepared meats",
                },

                new Category
                {
                    CategoryID = 7,
                    CategoryName = "Produce",
                    Description = "Dried fruit and bean curd",
                },

                new Category
                {
                    CategoryID = 8,
                    CategoryName = "Seafood",
                    Description = "Seaweed and fish",
                }
            };
        }

        private static List<Product> CreateProducts()
        {
            return new List<Product>
            {
                new Product
                {
                    ProductID = 1,
                    ProductName = "Chai",
                    CategoryID = 1,
                    QuantityPerUnit = "10 boxes x 20 bags",
                    UnitPrice = 18.0000m,
                    UnitsInStock = 39,
                    UnitsOnOrder = 0,
                    ReorderLevel = 10,
                    Discontinued = false
                },

                new Product
                {
                    ProductID = 2,
                    ProductName = "Chang",
                    CategoryID = 1,
                    QuantityPerUnit = "24 - 12 oz bottles",
                    UnitPrice = 19.0000m,
                    UnitsInStock = 17,
                    UnitsOnOrder = 40,
                    ReorderLevel = 25,
                    Discontinued = false
                },

                new Product
                {
                    ProductID = 24,
                    ProductName = "Guaraná Fantástica",
                    CategoryID = 1,
                    QuantityPerUnit = "12 - 355 ml cans",
                    UnitPrice = 4.5000m,
                    UnitsInStock = 20,
                    UnitsOnOrder = 0,
                    ReorderLevel = 0,
                    Discontinued = true
                },

                new Product
                {
                    ProductID = 77,
                    ProductName = "Original Frankfurter grüne Soße",
                    CategoryID = 2,
                    QuantityPerUnit = "12 boxes",
                    UnitPrice = 13.0000m,
                    UnitsInStock = 32,
                    UnitsOnOrder = 0,
                    ReorderLevel = 15,
                    Discontinued = false
                },

                new Product
                {
                    ProductID = 61,
                    ProductName = "Sirop d'érable",
                    CategoryID = 2,
                    QuantityPerUnit = "24 - 500 ml bottles",
                    UnitPrice = 28.5000m,
                    UnitsInStock = 113,
                    UnitsOnOrder = 0,
                    ReorderLevel = 25,
                    Discontinued = false
                },

                new Product
                {
                    ProductID = 63,
                    ProductName = "Vegie-spread",
                    CategoryID = 2,
                    QuantityPerUnit = "15 - 625 g jars",
                    UnitPrice = 43.9000m,
                    UnitsInStock = 24,
                    UnitsOnOrder = 0,
                    ReorderLevel = 5,
                    Discontinued = false
                },

                new Product
                {
                    ProductID = 16,
                    ProductName = "Pavlova",
                    CategoryID = 3,
                    QuantityPerUnit = "32 - 500 g boxes",
                    UnitPrice = 17.4500m,
                    UnitsInStock = 29,
                    UnitsOnOrder = 0,
                    ReorderLevel = 10,
                    Discontinued = false
                },

                new Product
                {
                    ProductID = 19,
                    ProductName = "Teatime Chocolate Biscuits",
                    CategoryID = 3,
                    QuantityPerUnit = "10 boxes x 12 pieces",
                    UnitPrice = 9.2000m,
                    UnitsInStock = 25,
                    UnitsOnOrder = 0,
                    ReorderLevel = 5,
                    Discontinued = false
                },

                new Product
                {
                    ProductID = 20,
                    ProductName = "Sir Rodney's Marmalade",
                    CategoryID = 3,
                    QuantityPerUnit = "30 gift boxes",
                    UnitPrice = 81.0000m,
                    UnitsInStock = 40,
                    UnitsOnOrder = 0,
                    ReorderLevel = 0,
                    Discontinued = false
                },

                new Product
                {
                    ProductID = 69,
                    ProductName = "Gudbrandsdalsost",
                    CategoryID = 4,
                    QuantityPerUnit = "10 kg pkg.",
                    UnitPrice = 36.0000m,
                    UnitsInStock = 26,
                    UnitsOnOrder = 0,
                    ReorderLevel = 15,
                    Discontinued = false
                },

                new Product
                {
                    ProductID = 71,
                    ProductName = "Flotemysost",
                    CategoryID = 4,
                    QuantityPerUnit = "10 - 500 g pkgs.",
                    UnitPrice = 21.5000m,
                    UnitsInStock = 26,
                    UnitsOnOrder = 0,
                    ReorderLevel = 0,
                    Discontinued = false
                },

                new Product
                {
                    ProductID = 72,
                    ProductName = "Mozzarella di Giovanni",
                    CategoryID = 4,
                    QuantityPerUnit = "24 - 200 g pkgs.",
                    UnitPrice = 34.8000m,
                    UnitsInStock = 14,
                    UnitsOnOrder = 0,
                    ReorderLevel = 0,
                    Discontinued = false
                },
                new Product
                {
                    ProductID = 22,
                    ProductName = "Gustaf's Knäckebröd",
                    CategoryID = 5,
                    QuantityPerUnit = "24 - 500 g pkgs.",
                    UnitPrice = 21.0000m,
                    UnitsInStock = 104,
                    UnitsOnOrder = 0,
                    ReorderLevel = 25,
                    Discontinued = false
                },

                new Product
                {
                    ProductID = 23,
                    ProductName = "Tunnbröd",
                    CategoryID = 5,
                    QuantityPerUnit = "12 - 250 g pkgs.",
                    UnitPrice = 9.0000m,
                    UnitsInStock = 61,
                    UnitsOnOrder = 0,
                    ReorderLevel = 25,
                    Discontinued = false
                },

                new Product
                {
                    ProductID = 53,
                    ProductName = "Perth Pasties",
                    CategoryID = 6,
                    QuantityPerUnit = "48 pieces",
                    UnitPrice = 32.8000m,
                    UnitsInStock = 0,
                    UnitsOnOrder = 0,
                    ReorderLevel = 0,
                    Discontinued = true
                },

                new Product
                {
                    ProductID = 54,
                    ProductName = "Tourtière",
                    CategoryID = 6,
                    QuantityPerUnit = "16 pies",
                    UnitPrice = 7.4500m,
                    UnitsInStock = 21,
                    UnitsOnOrder = 0,
                    ReorderLevel = 10,
                    Discontinued = false
                },

                new Product
                {
                    ProductID = 55,
                    ProductName = "Pâté chinois",
                    CategoryID = 6,
                    QuantityPerUnit = "24 boxes x 2 pies",
                    UnitPrice = 24.0000m,
                    UnitsInStock = 115,
                    UnitsOnOrder = 0,
                    ReorderLevel = 20,
                    Discontinued = false
                },

                new Product
                {
                    ProductID = 28,
                    ProductName = "Rössle Sauerkraut",
                    CategoryID = 7,
                    QuantityPerUnit = "25 - 825 g cans",
                    UnitPrice = 45.6000m,
                    UnitsInStock = 26,
                    UnitsOnOrder = 0,
                    ReorderLevel = 0,
                    Discontinued = true
                },

                new Product
                {
                    ProductID = 7,
                    ProductName = "Uncle Bob's Organic Dried Pears",
                    CategoryID = 7,
                    QuantityPerUnit = "12 - 1 lb pkgs.",
                    UnitPrice = 30.0000m,
                    UnitsInStock = 15,
                    UnitsOnOrder = 0,
                    ReorderLevel = 10,
                    Discontinued = false
                },

                new Product
                {
                    ProductID = 14,
                    ProductName = "Tofu",
                    CategoryID = 7,
                    QuantityPerUnit = "40 - 100 g pkgs.",
                    UnitPrice = 23.2500m,
                    UnitsInStock = 35,
                    UnitsOnOrder = 0,
                    ReorderLevel = 0,
                    Discontinued = false
                },

                new Product
                {
                    ProductID = 45,
                    ProductName = "Rogede sild",
                    CategoryID = 8,
                    QuantityPerUnit = "1k pkg.",
                    UnitPrice = 9.5000m,
                    UnitsInStock = 5,
                    UnitsOnOrder = 70,
                    ReorderLevel = 15,
                    Discontinued = false
                },

                new Product
                {
                    ProductID = 46,
                    ProductName = "Spegesild",
                    CategoryID = 8,
                    QuantityPerUnit = "4 - 450 g glasses",
                    UnitPrice = 12.0000m,
                    UnitsInStock = 95,
                    UnitsOnOrder = 0,
                    ReorderLevel = 0,
                    Discontinued = false
                },

                new Product
                {
                    ProductID = 58,
                    ProductName = "Escargots de Bourgogne",
                    CategoryID = 8,
                    QuantityPerUnit = "24 pieces",
                    UnitPrice = 13.2500m,
                    UnitsInStock = 62,
                    UnitsOnOrder = 0,
                    ReorderLevel = 20,
                    Discontinued = false
                }
            };
        }

        private List<Category> CreateCategoriesAndProducts()
        {
            List<Category> retVal = CreateCategories();

            List<Product> tempProducts = CreateProducts();

            Category parentCategory = null;

            foreach (Product p in tempProducts)
            {
                if (parentCategory == null || parentCategory.CategoryID != p.CategoryID)
                {
                    parentCategory = retVal.First<Category>(category => category.CategoryID == p.CategoryID);
                }

                parentCategory.AddProduct(p);
            }

            return retVal;
        }

        #endregion
    }

    public class Category
    {
        #region private fields

        private int categoryID;
        private string categoryName;
        private string description;

        private List<Product> products;

        #endregion

        #region public properties
        public int CategoryID
        {
            get
            {
                return this.categoryID;
            }
            set
            {
                if (this.categoryID != value)
                {
                    this.categoryID = value;
                }
            }
        }
       
        public string CategoryName
        {
            get
            {
                return this.categoryName;
            }
            set
            {
                if (this.categoryName != value)
                {
                    this.categoryName = value;
                }
            }
        }

        public string Description
        {
            get
            {
                return this.description;
            }
            set
            {
                if (this.description != value)
                {
                    this.description = value;
                }
            }
        }

        public IQueryable<Product> Products
        {
            get
            {
                if (products == null)
                {
                    products = new List<Product>();
                }

                return this.products.AsQueryable<Product>();
            }
        }

        #endregion

        #region public methods

        public void AddProduct(Product newProduct)
        {
            products.Add(newProduct);
        }

        #endregion
    }

    public class Product
    {
        #region private members

        private int productID;
        private string productName;
        private int categoryID;
        private string quantityPerUnit;
        private decimal unitPrice;
        private int unitsInStock;
        private int unitsOnOrder;
        private int reorderLevel;
        private bool discontinued;
        private int? myTestValue;

        #endregion

        #region public properties

        public int ProductID
        {
            get
            {
                return this.productID;
            }
            set
            {
                if (this.productID != value)
                {
                    this.productID = value;
                }
            }
        }

        public string ProductName
        {
            get
            {
                return this.productName;
            }
            set
            {
                if (this.productName != value)
                {
                    this.productName = value;
                }
            }
        }

        public int CategoryID
        {
            get
            {
                return this.categoryID;
            }
            set
            {
                if (this.categoryID != value)
                {
                    this.categoryID = value;
                }
            }
        }

        public string QuantityPerUnit
        {
            get
            {
                return this.quantityPerUnit;
            }
            set
            {
                if (this.quantityPerUnit != value)
                {
                    this.quantityPerUnit = value;
                }
            }
        }

        public decimal UnitPrice
        {
            get
            {
                return this.unitPrice;
            }
            set
            {
                if (value < 0)
                    throw new Exception("Negative numbers are not allowed.");

                this.unitPrice = value;
            }
        }

        public int UnitsInStock
        {
            get
            {
                return this.unitsInStock;
            }
            set
            {
                if (value < 0)
                    throw new Exception("Negative numbers are not allowed.");

                this.unitsInStock = value;
            }
        }

        
        public int UnitsOnOrder
        {
            get
            {
                return this.unitsOnOrder;
            }
            set
            {
                if (this.unitsOnOrder != value)
                {
                    this.unitsOnOrder = value;
                }
            }
        }

        public int ReorderLevel
        {
            get
            {
                return this.reorderLevel;
            }
            set
            {
                if (this.reorderLevel != value)
                {
                    this.reorderLevel = value;
                }
            }
        }

        public bool Discontinued
        {
            get
            {
                return this.discontinued;
            }
            set
            {
                if (this.discontinued != value)
                {
                    this.discontinued = value;
                }
            }
        }
        public int? MyTestValue
        {
            get
            {
                return this.myTestValue;
            }
            set
            {
                if (this.myTestValue != value)
                {
                    this.myTestValue = value;
                }
            }
        }
        #endregion
    }
}